


use std::fmt::Display;

use clap::{Parser, Subcommand, ValueEnum};

#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
pub struct Cli {


    #[command(subcommand)]
    pub comm: Option<Subc>,

    /// Debug level
    #[arg(short, long, default_value_t = 1)]
    pub debug: u8,
}

#[derive(Subcommand, Debug)]
pub enum Subc {

    Run {filename: String},
    Build {filename: String, output: String},
    View {

        filename: String,
       
        #[arg(short, long, default_value_t = FormatKind::FancyTable)]
        format: FormatKind

    },
    Debug {}

}

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum, Debug)]
pub enum FormatKind {
    FancyTable,
    Raw,
    BinSections,
    Serializable,
}


impl Display for FormatKind {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            FormatKind::FancyTable => write!(f, "fancy-table"),
            FormatKind::Raw => write!(f, "raw"),
            FormatKind::BinSections => write!(f, "memory"),
            FormatKind::Serializable => write!(f, "rust"),
        }
    }
}


