use super::registers::Register;

type Constant = i8; // 8 bits max, so it works.

// TODO: Use macros and a single reference for the ops.
#[derive(Debug)]
pub enum OP {
    NOP,
    ADD(Register, Register, Register),
    SUB(Register, Register, Register),
    AND(Register, Register, Register),
    XOR(Register, Register, Register),
    SLL(Register, Register, Register),

    SLI(Register, Constant),
    ADDI(Register, Constant),

    BEQ(Register, Register, Register),
    BGT(Register, Register, Register),
    JAL(Register, Register, Constant),

    LOAD(Register, Register, Register),
    STORE(Register, Register, Register),
    CALL(Register, Constant),
    HALT,
}

pub use OP::*;

/// Decodes a single binary operation.
pub fn decode(op: u16) -> OP {
    let opcode = op >> 12;
    let dest = ((op & 0x0F00) >> 8) as Register;
    let r1 = ((op & 0x00F0) >> 4) as Register;
    let r2 = (op & 0x000F) as Register;

    let c = Constant::from_be_bytes([(op & 0x00FF) as u8]);
    let c4 = Constant::from_be_bytes([(op & 0x000F) as u8]);

    // println!("opcode: {}", opcode);

    return match opcode {
        // todo: write a macro for every type (I-type, R-type)
        0b0000 => NOP,
        0b0001 => ADD(dest, r1, r2),
        0b0010 => SUB(dest, r1, r2),
        0b0011 => AND(dest, r1, r2),
        0b0100 => XOR(dest, r1, r2),
        0b0101 => SLL(dest, r1, r2),
        0b0110 => SLI(dest, c),
        0b0111 => ADDI(dest, c),
        0b1000 => BEQ(dest, r1, r2),
        0b1001 => BGT(dest, r1, r2),
        0b1010 => JAL(dest, r1, c4),
        0b1011 => todo!(),
        0b1100 => LOAD(dest, r1, r2),
        0b1101 => STORE(dest, r1, r2),
        0b1110 => CALL(dest, c),
        0b1111 => HALT,
        // opcode, by construction, is a binary 4 bits number.
        _ => unreachable!(),
    };
}
