use crate::cpu::registers::Word;

// TODO: abstract this into a trait and write different implementation
// of RAM, such as a growable cointainer/tree.

/// We'll define our RAM as a static array.
/// The maximum adressable memory is, right now, just 65kbit of memory.

pub const MAX_MEM: usize = 65535;
// pub const MAX_MEM: usize = 40;

#[derive(Debug)]
pub struct Ram {
    mem: [Word; MAX_MEM],
}

impl Default for Ram {
    fn default() -> Self {
        return Ram { mem: [0; MAX_MEM] };
    }
}

impl Ram {
    /// Gets the word at memory address i. Returns none if i is
    /// out of bounds.
    pub fn get(&self, i: Word) -> Option<Word> {
        if (i as usize) < MAX_MEM {
            return Some(self.mem[i as usize]);
        } else {
            return None;
        }
    }

    /// Writes val into memory address i. Returns none if i is
    /// out of bounds.
    pub fn write(&mut self, i: Word, val: Word) -> Option<()> {
        if (i as usize) < MAX_MEM {
            self.mem[i as usize] = val;
            return Some(());
        } else {
            return None;
        }
    }

    /// Returns a slice of memory from start to end address, inclusive.
    /// None is returned if the address is out of bounds.
    pub fn slice(&self, start: Word, end: Word) -> Option<&[Word]> {
        if (start as usize) < MAX_MEM && (end as usize) < MAX_MEM {
            return Some(&self.mem[(start as usize)..(end as usize)]);
        } else {
            return None;
        }
    }

    /// Writes an array of data directly into memory, starting from
    /// the "start" address.
    /// Returns None if the data exceeds memory.
    pub fn write_array(&mut self, data: &[Word], start: Word) -> Option<()> {
        if start as usize + data.len() < MAX_MEM {
            for i in 0..data.len() {
                self.mem[start as usize + i] = data[i];
            }
        } else {
            return None;
        }

        return Some(());
    }
}
