

use crate::assembler::{self, print_op};

use super::{Section, SectionContent};

/// Aaaand we don't know the size of the terminal.
/// We'll just pretend it is 80 characters.
/// It would be wise to be able to change it with 
/// a parameter. Later! #TODO

pub const TERMSIZE : u8 = 80;


pub fn to_table(secs: Vec<Section>) -> String {
    
    let mut res = String::new();

    let mut header = String::from("┌");

    for _ in 2..TERMSIZE {
        header.push('─');
    }

    header.push('┐');
    header.push('\n');
// ┏
// ┐
//
    res.push_str(&header);

    let sections_strings : Vec<String> = secs.into_iter().map(|x| draw_section(x)).collect();

    let cont = sections_strings.join(&draw_separator(TERMSIZE));

    res.push_str(&cont);

    let mut footer = String::from("└");

    for _ in 2..TERMSIZE {
        footer.push('─');
    }

    footer.push('┘');

    res.push_str(&footer);



    // for i in -40..40 {
    //     let p : i32 = (c as u32).try_into().unwrap();
    //      res.push_str(&format!("{}\n", char::from_u32((p + i).try_into().unwrap()).unwrap()));
    // }


    return res;

}


fn draw_section(sec: Section) -> String {

    let mut res = String::new();

    res.push_str(&format_line(&format!("{}", sec.name), 4, TERMSIZE));

    res.push_str(&draw_separator(TERMSIZE));

    match sec.content {
        SectionContent::Code(ops) => {
            // we just print ops again.
            // we could make this prettier.
            
            for op in ops {
                res.push_str(&format_line(&print_op(op), 8, TERMSIZE));

            }
        },
        SectionContent::CString(s) => res.push_str(&s),
        SectionContent::CVec() => todo!(),
    }


    return res;



}

/// Formats a line in a box, with said indendation.
fn format_line(s: &str, indentation: u8, len: u8) -> String {
    let mut res = String::from("│");

    // TODO: UTF8 GRAPHEMES???;

    for _ in 1..indentation {
        res.push(' ');
    }

    res.push_str(s);

    // if ASCII, or even extended ascii are concerned,
    // this works. But we need to get the number of graphemes, which can be tricky
    // in utf-8.

    for _ in res.len()..(len + 1) as usize {
        res.push(' ');
    }

    res.push_str("│\n");


    return res;
}


/// Draws a horizontal line of specified length.
///
fn draw_separator(len: u8) -> String {

    let mut res = String::from("├");

    for _ in 2..len {
        res.push('─');
    }

    res.push('┤');
    res.push('\n');

    return res;

}




// fn wrap_in_bars(&str, indentation: u8) -> String {
//
//
//
// }

// ┄
// ┅
// ┆
// ┇
// ┈
// ┉
// ┊
// ┋
// ┌
// ┍
// ┎
// ┏
// ┐
// ┑
// ┒
// ┓
// └
// ┕
// ┖
// ┗
// ┘
// ┙
// ┚
// ┛
// ├
// ┝
// ┞
// ┟
// ┠
// ┡
// ┢
// ┣
// ┤
// ┥
// ┦
// ┧
// ┨
// ┩
// ┪
// ┫
// ┬
// ┭
// ┮
// ┯
// ┰
// ┱
// ┲
// ┳
// ┴
// ┵
// ┶
// ┷
// ┸
// ┹
// ┺
// ┻
// ┼
// ┽
// ┾
// ┿
// ╀
// ╁
// ╂
// ╃
// ╄
// ╅
// ╆
// ╇
// ╈
// ╉
// ╊
// ╋
// ╌
// ╍
// ╎
// ╏
// ═
// ║
// ╒
// ╓
